try:
    FileNotFoundError
except NameError:
    FileNotFoundError = IOError
import pytest
from rev_assets import RevAssets, AssetNotFound


def test_default_url():
    rev = RevAssets(manifest='tests-manifest.json')
    asset = 'styles/home.css'
    expected = '/static/styles/home-2b0339133a.css'
    assert rev.asset_url(asset) == expected


def test_base_url():
    rev = RevAssets(
        base_url='//cdn.amazon.com',
        manifest='tests-manifest.json'
    )
    asset = 'styles/home.css'
    expected = '//cdn.amazon.com/styles/home-2b0339133a.css'
    assert rev.asset_url(asset) == expected


def test_base_url_too_many_slashes():
    rev = RevAssets(
        base_url='//cdn.amazon.com/',
        manifest='tests-manifest.json'
    )
    asset = '/styles/home.css'
    expected = '//cdn.amazon.com/styles/home-2b0339133a.css'
    assert rev.asset_url(asset) == expected


def test_base_url_to_few_slashes():
    rev = RevAssets(
        base_url='',
        manifest='tests-manifest.json'
    )
    asset = 'styles/home.css'
    expected = '/styles/home-2b0339133a.css'
    assert rev.asset_url(asset) == expected


def test_base_url_local():
    rev = RevAssets(
        base_url='/static/',
        manifest='tests-manifest.json'
    )
    asset = 'styles/home.css'
    expected = '/static/styles/home-2b0339133a.css'
    assert rev.asset_url(asset) == expected

    asset = 'scripts/home.js'
    expected = '/static/scripts/home-0ec3e34646.js'
    assert rev.asset_url(asset) == expected


def test_dont_reload():
    rev = RevAssets(manifest='tests-manifest.json', reload=False)
    asset = 'styles/home.css'
    expected = '/static/styles/home-2b0339133a.css'
    assert rev.asset_url(asset) == expected

    new = 'foobar'
    rev._load_manifest = lambda: {asset: new}
    assert rev.asset_url(asset) == expected


def test_reload():
    rev = RevAssets(
        base_url='/static/',
        manifest='tests-manifest.json',
        reload=True
    )
    asset = 'styles/home.css'
    expected = '/static/styles/home-2b0339133a.css'
    assert rev.asset_url(asset) == expected

    new = 'whatever'
    rev._load_manifest = lambda: {asset: new}
    assert rev.asset_url(asset) == '/static/' + new


def test_no_manifest():
    rev = RevAssets()
    with pytest.raises(FileNotFoundError):
        rev.asset_url('styles/home.css')


def test_no_asset_quiet():
    rev = RevAssets(manifest='tests-manifest.json')
    asset = 'no can do'
    rev.asset_url(asset) == ''


def test_no_asset_loud():
    rev = RevAssets(manifest='tests-manifest.json', quiet=False)
    asset = 'no can do'
    with pytest.raises(AssetNotFound):
        rev.asset_url(asset)



"""
===========================
RevAssets
===========================

Makes possible for python web apps to work with hashed static assets
generated by other tools like Gulp or Webpack.

It does so by reading the manifest generated by the revision tool.

"""
import json
import io


__version__ = '1.0.3'


class AssetNotFound(Exception):
    pass


class RevAssets(object):
    """
    Map the source -> hashed assets

    :param base_url: From where the hashed assets are served.
    :param reload: Reload the manifest each time an asset is requested.
    :param manifest: Path and filename of the manifest file.
    :param quiet: If False, a missing asset will raise an exception

    """

    def __init__(self, base_url='/static', reload=False,
                 manifest='manifest.json', quiet=True):
        self.base_url = base_url.rstrip('/')
        self.reload = reload
        self.manifest = manifest
        self.assets = {}
        self.quiet = quiet

    def _load_manifest(self):
        with io.open(self.manifest, 'rt', encoding='utf-8') as mf:
            return json.loads(mf.read())

    def asset_url(self, asset):
        if not self.assets or self.reload:
            self.assets = self._load_manifest()
        asset = asset.strip('/')
        path = self.assets.get(asset)
        if not path:
            if self.quiet:
                return ''
            msg = 'Asset file {!r} not found'.format(asset)
            raise AssetNotFound(msg)

        return '{}/{}'.format(
            self.base_url,
            path.lstrip('/'),
        )
